/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* read and parse configuration
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 05/01/2000
*/

#include "config.h"
#include "struct.h"

extern int read_from_sock(char **line, int sock_fd, BOOL wait);
extern int print_log(char *str);

boxlist   *config_boxlist;
decllist  *config_decllist  = NULL;
aliaslist *config_aliaslist = NULL;

FILE *config_logfile;
char config_ownbbs[7];
char *config_mailserv;
char *config_mxspool;
char *config_mxsender;
char *config_fwdprg;
char *config_boxsock;
char *config_expdir;
char *config_impdir;
char *config_faildir;
char *config_copydir;
char *config_tmpdir;
char *config_decoder;
char *config_encoder;
char *config_mimeenc[5];
char *config_mimedec[5];
char *config_mimecon[5];
char *config_mimesuf[5];
int  config_verbose;
int  decltyp = -1;

/* copy char array */
char *config_copy_char(char *old)
{
  char *new;
  
  if(! old) return NULL;
  
  new = (char *) malloc(sizeof(char) * (strlen(old) + 1));
  if(! new) {
    perror("malloc");
    exit(1);
  }
  
  strcpy(new, old);
  
  return new;                                                                   }
  
/* replace "~" with homedir and "$" with username */
char *config_parse_str(char *str)
{
  char *new;
  char *pat;
  char *home, *user;
  
  home = NULL;
  user = NULL;

  new = config_copy_char(str);

  while((pat = strchr(new, '~'))) {
    if(! home) {
      home = getenv("HOME");
      if(! home) {
	fprintf(stderr, "FATAL: $HOME not set, but \"~\" used\n");
	exit(1);
      }
    }
    new = (char *) realloc(new, sizeof(char) * (strlen(new) + strlen(home)));
    if(! new) {
      perror("realloc");
      exit(1);
    }
    
    pat = strchr(new, '~');
    memmove(pat+strlen(home), pat+1, strlen(pat));
    memmove(pat, home, strlen(home));
  }

  while((pat = strchr(new, '$'))) {
    if(! user) {
      user = getenv("LOGNAME");
      if(! user)
	user = getlogin();
      if(! user) {
	fprintf(stderr, "FATAL: username unknown, but \"$\" used\n");
	exit(1);
      }
    }
    new = (char *) realloc(new, sizeof(char) * (strlen(new) + strlen(user)));
    if(! new) {
      perror("realloc");
      exit(1);
    }

    pat = strchr(new, '$');
    memmove(pat+strlen(user), pat+1, strlen(pat));
    memmove(pat, user, strlen(user));
  }

  return new;
}

/* returns (new allocated) string with directory-path (without filename)
   "/dir1/dir2/dir3/file" -> "/dir1/dir2/dir3/" (with ending slash!) */
char *config_extract_dirname(char *file_str)
{
  int i;
  char *ret_str;
 
  if((! file_str) || (strlen(file_str) < 1)) return(NULL);
  for(i=strlen(file_str)-1;i>=0;i--) if(*(file_str+i) == '/') break;
 
  /* only nessasary if more than one slash found (!= root-dir) */
  if(i < 1) return(NULL);
 
  i += 2;
  ret_str = (char *) malloc(sizeof(char) * i);
  if(! ret_str) {
    perror("malloc");
    exit(1);
  }
 
  memcpy(ret_str, file_str, i-1);
  *(ret_str + (i-1)) = 0;
  return(ret_str);
}
 
/* create needed subdirectorys for file, mode=TRUE: cut ending file */
int config_create_sub_dirs(char *filename, BOOL mode)
{
  char *dir_str, *error_str;
  int i;
 
  if(mode) {
    dir_str = config_extract_dirname(filename);
  } else {
    if(! filename) return 0;
    dir_str = (char *) malloc(sizeof(char) * (strlen(filename) +2));
    if(! dir_str) {
      perror("malloc");
      exit(1);
    }
    strcpy(dir_str, filename);
    if(*(dir_str + strlen(dir_str) -1) != '/') strcat(dir_str, "/");
  }
  if(! dir_str) return(0);
 
  i = 1;
  while(1) {
    if(*(dir_str + i) == 0) break;
    if(*(dir_str + i) == '/') {
      *(dir_str + i) = 0;
      if(access(dir_str, F_OK) != 0) {
        /* subdir doesn't exists, try to create */
        if(mkdir(dir_str, 0777)) {
          /* failed... :-( */
          error_str = (char *) malloc(sizeof(char) * (strlen(dir_str)+80));
          if(! error_str) {
            perror("malloc");
            exit(1);
          }
          sprintf(error_str, "creating of directory \"%s\" failed", dir_str);
	  print_log(error_str);
          free(error_str);
          free(dir_str);
          return(1);
        }
      }
      *(dir_str + i) = '/';
    }
    i++;
  }
  free(dir_str);
  return(0);
}

int config_readrc(char *rcfile)
{
  int rc;
  char *file_name;
  char *log_file;
  int line;
  char *inp_line;
  boxlist   *bl_new;
  aliaslist *al_new;
  char *pat;
  int i;

  log_file = NULL;

  if(! rcfile) {
    file_name = config_parse_str(SFTOOLRC);
  } else {
    file_name = config_copy_char(rcfile);
  }

  rc = open(file_name, O_RDONLY);
  if(rc < 0) {
    fprintf(stderr, "could not open rcfile \"%s\"\n", file_name);
    perror("open");
    exit(1);
  }

  free(file_name);

  config_boxlist = NULL;
  strcpy(config_ownbbs, OWNBBS);
  config_mailserv   = config_parse_str(MAILSERV);
  config_mxspool    = config_parse_str(MXSPOOL);
  config_mxsender   = config_parse_str(MXSENDER);
  config_fwdprg     = config_parse_str(FWDPRG);
  config_boxsock    = config_parse_str(BOXSOCK);
  config_expdir     = config_parse_str(EXPDIR);
  config_impdir     = config_parse_str(IMPDIR);
  config_faildir    = config_parse_str(FAILDIR);
  config_copydir    = config_parse_str(COPYDIR);
  config_tmpdir     = config_parse_str(TMPDIR);
  config_encoder    = config_parse_str(ENCODER);
  config_decoder    = config_parse_str(DECODER);
  config_mimeenc[0] = config_parse_str(MIMEENC1);
  config_mimedec[0] = config_parse_str(MIMEDEC1);
  config_mimecon[0] = config_parse_str(MIMECON1);
  config_mimesuf[0] = config_parse_str(MIMESUF1);
  config_mimeenc[1] = config_parse_str(MIMEENC2);
  config_mimedec[1] = config_parse_str(MIMEDEC2);
  config_mimecon[1] = config_parse_str(MIMECON2);
  config_mimesuf[1] = config_parse_str(MIMESUF2);
  config_mimeenc[2] = config_parse_str(MIMEENC3);
  config_mimedec[2] = config_parse_str(MIMEDEC3);
  config_mimecon[2] = config_parse_str(MIMECON3);
  config_mimesuf[2] = config_parse_str(MIMESUF3);
  config_mimeenc[3] = config_parse_str(MIMEENC4);
  config_mimedec[3] = config_parse_str(MIMEDEC4);
  config_mimecon[3] = config_parse_str(MIMECON4);
  config_mimesuf[3] = config_parse_str(MIMESUF4);
  config_mimeenc[4] = config_parse_str(MIMEENC5);
  config_mimedec[4] = config_parse_str(MIMEDEC5);
  config_mimecon[4] = config_parse_str(MIMECON5);
  config_mimesuf[4] = config_parse_str(MIMESUF5);
  
  log_file          = config_parse_str(LOGFILE);

  line = 0;
  while(read_from_sock(&inp_line, rc, FALSE) == 0) {
    line++;
    if(inp_line) {
      file_name = strrchr(inp_line, '\n');
      if(file_name) *file_name = 0;
      file_name = strrchr(inp_line, '\r');
      if(file_name) *file_name = 0;
    }
    if(inp_line && *inp_line && *inp_line != '#') {
      if(strncasecmp(inp_line, "OWNBBS=", 7) == 0) {
	if(strlen(inp_line+7) > 6) {
	  fprintf(stderr, "OWNBBS too long in line %d\n", line);
	  exit(1);
	}
	strcpy(config_ownbbs, inp_line+7);
      } else if(strncasecmp(inp_line, "LOGFILE=", 8) == 0) {
	if(log_file) free(log_file);
	log_file = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "MAILSERV=", 9) == 0) {
	if(config_mailserv) free(config_mailserv);
	config_mailserv = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MXSPOOL=", 8) == 0) {
	if(config_mxspool) free(config_mxspool);
	config_mxspool = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "MXSENDER=", 9) == 0) {
	if(config_mxsender) free(config_mxsender);
	config_mxsender = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "FWDPRG=", 7) == 0) {
	if(config_fwdprg) free(config_fwdprg);
	config_fwdprg = config_parse_str(inp_line + 7);
      } else if(strncasecmp(inp_line, "BOXSOCK=", 8) == 0) {
	if(config_boxsock) free(config_boxsock);
	config_boxsock = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "EXPDIR=", 7) == 0) {
	if(config_expdir) free(config_expdir);
	config_expdir = config_parse_str(inp_line + 7);
      } else if(strncasecmp(inp_line, "IMPDIR=", 7) == 0) {
	if(config_impdir) free(config_impdir);
	config_impdir = config_parse_str(inp_line + 7);
      } else if(strncasecmp(inp_line, "FAILDIR=", 8) == 0) {
	if(config_faildir) free(config_faildir);
	config_faildir = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "COPYDIR=", 8) == 0) {
	if(config_copydir) free(config_copydir);
	config_copydir = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "TMPDIR=", 7) == 0) {
	if(config_tmpdir) free(config_tmpdir);
	config_tmpdir = config_parse_str(inp_line + 7);
      } else if(strncasecmp(inp_line, "ENCODER=", 8) == 0) {
	if(config_encoder) free(config_encoder);
	config_encoder = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "DECODER=", 8) == 0) {
	if(config_decoder) free(config_decoder);
	config_decoder = config_parse_str(inp_line + 8);
      } else if(strncasecmp(inp_line, "MIMEENC1=", 9) == 0) {
	if(config_mimeenc[0]) free(config_mimeenc[0]);
	config_mimeenc[0] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEDEC1=", 9) == 0) {
	if(config_mimedec[0]) free(config_mimedec[0]);
	config_mimedec[0] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMECON1=", 9) == 0) {
	if(config_mimecon[0]) free(config_mimecon[0]);
	config_mimecon[0] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMESUF1=", 9) == 0) {
	if(config_mimesuf[0]) free(config_mimesuf[0]);
	config_mimesuf[0] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEENC2=", 9) == 0) {
        if(config_mimeenc[1]) free(config_mimeenc[1]);
        config_mimeenc[1] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEDEC2=", 9) == 0) {
        if(config_mimedec[1]) free(config_mimedec[1]);
        config_mimedec[1] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMECON2=", 9) == 0) {
        if(config_mimecon[1]) free(config_mimecon[1]);
        config_mimecon[1] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMESUF2=", 9) == 0) {
        if(config_mimesuf[1]) free(config_mimesuf[1]);
        config_mimesuf[1] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEENC3=", 9) == 0) {
        if(config_mimeenc[2]) free(config_mimeenc[2]);
        config_mimeenc[2] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEDEC3=", 9) == 0) {
        if(config_mimedec[2]) free(config_mimedec[2]);
        config_mimedec[2] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMECON3=", 9) == 0) {
        if(config_mimecon[2]) free(config_mimecon[2]);
        config_mimecon[2] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMESUF3=", 9) == 0) {
        if(config_mimesuf[2]) free(config_mimesuf[2]);
        config_mimesuf[2] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEENC4=", 9) == 0) {
        if(config_mimeenc[3]) free(config_mimeenc[3]);
        config_mimeenc[3] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEDEC4=", 9) == 0) {
        if(config_mimedec[3]) free(config_mimedec[3]);
        config_mimedec[3] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMECON4=", 9) == 0) {
        if(config_mimecon[3]) free(config_mimecon[3]);
        config_mimecon[3] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMESUF4=", 9) == 0) {
        if(config_mimesuf[3]) free(config_mimesuf[3]);
        config_mimesuf[3] = config_parse_str(inp_line + 9);    
      } else if(strncasecmp(inp_line, "MIMEENC5=", 9) == 0) {
        if(config_mimeenc[4]) free(config_mimeenc[4]);
        config_mimeenc[4] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMEDEC5=", 9) == 0) {
        if(config_mimedec[4]) free(config_mimedec[4]);
        config_mimedec[4] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMECON5=", 9) == 0) {
        if(config_mimecon[4]) free(config_mimecon[4]);
        config_mimecon[4] = config_parse_str(inp_line + 9);
      } else if(strncasecmp(inp_line, "MIMESUF5=", 9) == 0) {
        if(config_mimesuf[4]) free(config_mimesuf[4]);
        config_mimesuf[4] = config_parse_str(inp_line + 9);   
      } else if(strncasecmp(inp_line, "ALIAS=", 6) == 0) {
	file_name = strchr(inp_line+6, ',');
	if(! file_name) {
	  fprintf(stderr, "missing alias eMail-address in line %d\n", line);
	  exit(1);
	}
	*file_name = 0;
	if(strlen(inp_line+6) > 6) {
	  fprintf(stderr, "wrong boxname \"%s\" in line %d\n", inp_line+6, line);
	  exit(1);
	}
	al_new = (aliaslist *) malloc(sizeof(aliaslist));
	if(! al_new) {
	  perror("malloc");
	  exit(1);
	}
	al_new->next     = config_aliaslist;
	config_aliaslist = al_new;
	strcpy(al_new->bbs, inp_line+6);
	for(i=0;i<6;i++) al_new->bbs[i] = toupper(al_new->bbs[i]);

	file_name++;
	if(! *file_name) {
	  fprintf(stderr, "empty aliasfield for box %s in line %d\n", al_new->bbs, line);
	  exit(1);
	}
	al_new->alias = config_parse_str(file_name);
      } else if(strncasecmp(inp_line, "BOX=", 4) == 0) {
	file_name = strchr(inp_line+4, ',');
	if(! file_name) {
	  fprintf(stderr, "missing protokoll, mime and eMail-address in line %d\n", line);
	  exit(1);
	}
	*file_name = 0;
	if(strlen(inp_line+4) > 6) {
	  fprintf(stderr, "wrong boxname \"%s\" in line %d\n", inp_line+4, line);
	  exit(1);
	}
	bl_new = (boxlist *) malloc(sizeof(boxlist));
	if(! bl_new) {
	  perror("malloc");
	  exit(1);
	}
	bl_new->next   = config_boxlist;
	config_boxlist = bl_new;
	strcpy(bl_new->bbs, inp_line+4);
	for(i=0;i<6;i++) bl_new->bbs[i] = toupper(bl_new->bbs[i]);
	
	file_name++;
	pat = strchr(file_name, ',');
	if(! pat) {
	  fprintf(stderr, "missing Mime and eMail-address in line %d\n", line);
	  exit(1);
	}
	*pat = 0;
	bl_new->protokoll = atoi(file_name);
	if(bl_new->protokoll < 1 || bl_new->protokoll > 3) {
	  fprintf(stderr, "invalid protokoll %d in line %d\n", bl_new->protokoll, line);
	  exit(1);
	}

	pat++;
	file_name = pat;
	pat = strchr(pat, ',');
	if(! pat) {
	  fprintf(stderr, "missing eMail-address in line %d\n", line);
	  exit(1);
	}
	*pat = 0;
	bl_new->mime = atoi(file_name);
	if(bl_new->mime < 0 || bl_new->mime > 5) {
	  fprintf(stderr, "invalid mime %d in line %d\n", bl_new->mime, line);
	  exit(1);
	}

	pat++;
	if(! *pat) {
	  fprintf(stderr, "missing eMail-address in line %d\n", line);
	  exit(1);
	}
	bl_new->email = config_parse_str(pat);
      } else {
	fprintf(stderr, "invalid config \"%s\" in line %d\n", inp_line, line);
	exit(1);
      }
    }
    if(inp_line) free(inp_line);
  }

  if(log_file && (strcmp(log_file, "0") != 0)) {
    config_create_sub_dirs(log_file, TRUE);
    config_logfile = fopen(log_file, "a");
    if(! config_logfile) {
      fprintf(stderr, "warning: could not open logfile \"%s\"\n", log_file);
    }
  }

  if(log_file) free(log_file);
  if(! config_logfile) config_logfile = stderr;

  return 0;
}

int config_get_protokol(char *str)
{
  boxlist *curr;

  curr = config_boxlist;
  while(curr) {
    if(strcasecmp(str, curr->bbs) == 0) return curr->protokoll;
    curr = curr->next;
  }

  return 0;
}

int config_get_mime(char *str)
{
  boxlist *curr;
  
  curr = config_boxlist;
  while(curr) {
    if(strcasecmp(str, curr->bbs) == 0) return curr->mime;
    curr = curr->next;
  }
  
  return -1;
}

void config_get_box(char *str, char *mail)
{
  boxlist   *curr_bl;
  aliaslist *curr_al;
  char sub_str[50];

  curr_bl = config_boxlist;
  while(curr_bl) {
    if(strcasecmp(mail, curr_bl->email) == 0) {
      strcpy(str, curr_bl->bbs);
      return;
    }
    curr_bl = curr_bl->next;
  }

  curr_al = config_aliaslist;
  while(curr_al) {
    if(strcasecmp(mail, curr_al->alias) == 0) {
      strcpy(str, curr_al->bbs);
      sprintf(sub_str, "Mailbox %s was detected by alias", str);
      print_log(sub_str);
      if(! config_get_protokol(str)) {
	sprintf(sub_str, "...but mailbox %s is unknown", str);
	print_log(sub_str);
	str = NULL;
      }
      return;
    }
    curr_al = curr_al->next;
  }
}

int config_include_box(char *box)
{
  decllist *new;

  if(decltyp == -1) {
    decltyp = 0;
  } else if(decltyp != 0) {
    fprintf(stderr, "excludelist already build, cannot include\n");
    return 1;
  }

  if(strlen(box) > 6) {
    fprintf(stderr, "boxname for includelist is illegal\n");
    return 1;
  }

  new = (decllist *) malloc(sizeof(decllist));
  if(! new) {
    perror("malloc");
    exit(1);
  }

  new->next       = config_decllist;
  config_decllist = new;
  strcpy(new->bbs, box);

  return 0;
}

int config_exclude_box(char *box)
{
  decllist *new;

  if(decltyp == -1) {
    decltyp = 1;
  } else if(decltyp != 1) {
    fprintf(stderr, "includelist already build, cannot include\n");
    return 1;
  }

  if(strlen(box) > 6) {
    fprintf(stderr, "boxname for excludelist is illegal\n");
    return 1;
  }

  new = (decllist *) malloc(sizeof(decllist));
  if(! new) {
    perror("malloc");
    exit(1);
  }

  new->next       = config_decllist;
  config_decllist = new;
  strcpy(new->bbs, box);

  return 0;
}

int config_box_action_allowed(char *box)
{
  decllist *curr;
  int found;

  if(decltyp == -1) return 1;

  found = 0;
  curr  = config_decllist;

  while(curr) {
    if(strcasecmp(box, curr->bbs) == 0) {
      found = 1;
      break;
    }
    curr = curr->next;
  }

  if(found) {
    return 1 - decltyp;
  } else {
    return decltyp;
  }
}

