/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* doing import
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 1/11/1999
*/

#include "config.h"

#define MAXLEN 512

extern char *config_faildir;
extern char *config_impdir;
extern char *config_decoder;
extern char *config_fwdprg;
extern char *config_boxsock;
extern char *config_expdir;
extern char *config_mimedec[];

extern int print_log(char *str);
extern int spool_get_mails(void);
extern int config_create_sub_dirs(char *filename, BOOL mode);
extern int spool_copy_file(char *inputfile, char *outputdir, char *str);
extern char *spool_find_unique(char *outputdir, char *str);
extern int shell_run_cmd(char *cmd, char *pat1, char *pat2);
extern int config_get_protokol(char *str);
extern int config_get_mime(char *);
extern char *config_copy_char(char *old);
extern void config_get_box(char *str, char *mail);

int import_main(void)
{
  print_log("starting import: looking for new mails");
  if(spool_get_mails()) {
    print_log("import failed");
    return 1;
  }
  
  print_log("importing finished");
  return 0;
}

int import_file_import(char *file)
{
  FILE *in_fd;
  char frombbs[7];
  int protokol;
  char buffer[MAXLEN];
  char tmpbuf[MAXLEN];
  char *str, *pat;
  char *export;
  int ex_stat, i;
  int mime;
  char *email_sender;
  
  if(! file) return 1;

  frombbs[0]   = 0;
  protokol     = 0;
  mime         = -1;
  email_sender = NULL;

  in_fd = fopen(file, "r");
  if(! in_fd) return 1;

  while(1) {
    fgets(buffer, MAXLEN-1, in_fd);
    if(feof(in_fd)) break;

    str = strchr(buffer, '\n');
    if(str) *str = 0;
    str = strchr(buffer, '\r');
    if(str) *str = 0;
    
    if(strncmp(buffer, "From: ", 6) == 0) {
      email_sender = buffer + 6;
      str = strchr(email_sender, '@');
      if(str) {
	pat = strchr(str, ' ');
	if(pat) *pat = 0;
	pat = strchr(str, '>');
	if(pat) *pat = 0;
	pat = strrchr(email_sender, ' ');
	if(pat) email_sender = pat+1;
	pat = strrchr(email_sender, '<');
	if(pat) email_sender = pat+1;

	email_sender = config_copy_char(email_sender);
	sprintf(buffer, "FROM \"%s\"", email_sender);
	print_log(buffer);
      }
    } else {
      strcpy(tmpbuf, buffer);
      /* This string is malformed in a quoted-printable mail, so
	 check this first */
      str = strstr(buffer, "Export-Mailbox=3D");
      if(str) {
	memmove(str+15, str+17, strlen(str)-16);
      }
      str = strstr(buffer, "Export-Mailbox=");
      if(str) {
	pat = strchr(str+15, ' ');
	if(pat) *pat = 0;
	pat = strchr(str+15, ';');
	if(pat) *pat = 0;
	pat = strchr(str+15, ',');
	if(pat) *pat = 0;
	pat = strchr(str+15, '=');
	if(pat) *pat = 0;
	pat = str+15;
	if(strlen(pat) <= 6) {
	  strcpy(frombbs, pat);
	}
      }
      
      /* do it for mime */
      str = strstr(tmpbuf, "Export-Mime=3D");
      if(str) {
	memmove(str+12, str+14, strlen(str)-13);
      }
      str = strstr(tmpbuf, "Export-Mime=");
      if(str) {
	pat = strchr(str+12, ' ');
	if(pat) *pat = 0;
	pat = strchr(str+12, ';');
	if(pat) *pat = 0;
	pat = strchr(str+12, ',');
	if(pat) *pat = 0;
	pat = strchr(str+12, '=');
	if(pat) *pat = 0;
	pat = str+12;
	mime = atoi(pat);
      }
      
      /* same with protokoll */
      str = strstr(tmpbuf, "Export-Protokoll=3D");
      if(str) {
	memmove(str+17, str+19, strlen(str)-18);
      }
      str = strstr(tmpbuf, "Export-Protokoll=");
      if(str) {
	pat = strchr(str+17, ' ');
	if(pat) *pat = 0;
	pat = strchr(str+17, ';');
	if(pat) *pat = 0;
	pat = strchr(str+17, ',');
	if(pat) *pat = 0;
	pat = strchr(str+17, '=');
	if(pat) *pat = 0;
	pat = str+17;
	if(strcasecmp(pat, "F6FBB") == 0 || strcasecmp(pat, "FBB") == 0 ||
	   strcasecmp(pat, "1") == 0) {
	  protokol = 1;
	} else if(strcasecmp(pat, "WINBOX") == 0 || strcasecmp(pat, "WINGT") == 0 ||
		  strcasecmp(pat, "2") == 0 || strcasecmp(pat, "GT") == 0) {
	  protokol = 2;
	} else if(strcasecmp(pat, "DPBOX") == 0 || strcasecmp(pat, "DP") == 0 ||
		  strcasecmp(pat, "3") == 0 || strcasecmp(pat, "DIGIPOINT") == 0) {
	  protokol = 3;
	}
      }
      
      if(protokol && frombbs[0] && (mime != -1)) break;
    }
  }

  fclose(in_fd);

  if(! frombbs[0]) {
    print_log("senderbox unknown, try to find out from list");
    config_get_box(frombbs, email_sender);
  }

  if(email_sender) free(email_sender);

  if(! frombbs[0]) {
    print_log("senderbox still unknown, cannot import");
    spool_copy_file(file, config_faildir, "impfail");
    return 1;
  }

  if(! protokol) {
    print_log("missing protokoll, try to find out from list");
    protokol = config_get_protokol(frombbs);
    if(! protokol) {
      print_log("protokoll still unknown, cannot import");
      spool_copy_file(file, config_faildir, "impfail");
      return 1;
    }
  }

  if(mime == -1) {
    print_log("missing mime, try to find out from list");
    mime = config_get_mime(frombbs);
    if(mime == -1) {
      print_log("mime still unknown, cannot import");
      spool_copy_file(file, config_faildir, "impfail");
      return 1;
    }
  }

  sprintf(buffer, "got import from %s [typ=%d, mime=%d]", frombbs, protokol, mime);
  print_log(buffer);

  export = spool_find_unique(config_impdir, "import");
  if(! export) {
    print_log("no exportname created?!");
    spool_copy_file(file, config_faildir, "impfail");
    return 1;
  }

  if(mime == 0) ex_stat = shell_run_cmd(config_decoder, file, export);
  else          ex_stat = shell_run_cmd(config_mimedec[mime-1], file, export);
  if(ex_stat) {
    sprintf(buffer, "decoding import failed with code %d", ex_stat);
    spool_copy_file(file, config_faildir, "impfail");
    print_log(buffer);
    free(export);
    return 1;
  }

  if(access(export, F_OK)) {
    print_log("decoder finished, but exportfile is missing, abort");
    spool_copy_file(file, config_faildir, "impfail");
    free(export);
    return 1;
  }

  print_log("import successfully decoded");

  if(! config_fwdprg) {
    print_log("no forwardprogram defined, abort");
    spool_copy_file(export, config_faildir, frombbs);
    free(export);
    return 1;
  }

  if(! config_boxsock) {
    print_log("missing boxsocket, abort");
    spool_copy_file(export, config_faildir, frombbs);
    free(export);
    return 1;
  }
  
  str = (char *) malloc(sizeof(char) * (strlen(config_fwdprg) + strlen(config_boxsock) 
					+ strlen(config_expdir) + strlen(export) + 80));
  if(! str) {
    perror("malloc");
    exit(1);
  }

  sprintf(str, "%s %s -s %s ", config_fwdprg, frombbs, config_boxsock);
  switch(protokol) {
    case 1:
      strcat(str, "-fbb ");
      break;
    case 2:
      strcat(str, "-gt ");
      break;
    case 3:
      strcat(str, "-dp ");
      break;
    default:
      strcat(str, "-fbb ");
  }
  
  strcat(str, export);

  switch(config_get_protokol(frombbs)) {
    case 1:
      strcat(str, " -fbb ");
      break;
    case 2:
      strcat(str, " -gt ");
      break;
    case 3:
      strcat(str, " -dp ");
      break;
    default:
      print_log("warning, box is not defined in sftoolrc, using fbb as outputformat");
      strcat(str, " -fbb ");
      break;
  }

  for(i=0;i<7;i++) frombbs[i] = toupper(frombbs[i]);
  strcat(str, config_expdir);
  strcat(str, "/");
  strcat(str, frombbs);
  strcat(str, ".exp");

  config_create_sub_dirs(config_expdir, FALSE);

  ex_stat = shell_run_cmd(str, NULL, NULL);
  free(str);
  if(ex_stat) {
    sprintf(buffer, "importing failed with code %d", ex_stat);
    print_log(buffer);
    spool_copy_file(export, config_faildir, frombbs);
    unlink(export);
    free(export);
    return 1;
  }
  
  unlink(export);
  free(export);
  print_log("import successfully finished");

  return 0;
}

