/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* lockfile handling
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 27/9/1999
*/

#include "config.h"

extern int print_log(char *str);

int lock_lock_file(char *filename)
{
  char *lockfile;
  FILE *lckdes;
  pid_t lckpid;

  if(! filename) return 0;

  lockfile = (char *) malloc(sizeof(char) * (strlen(filename) + 6));
  if(! lockfile) {
    perror("malloc");
    exit(1);
  }

  sprintf(lockfile, "%s.lock", filename);

  lckdes = fopen(lockfile, "r");
  if(lckdes) {     /* lockfile exists */
    if(fscanf(lckdes, "%d", &lckpid) == 1) {
      if(kill(lckpid, 0) == -1 && errno == ESRCH) {        /* process is not longer existing */
	print_log("removing stale lockfile.");
	fclose(lckdes);
	unlink(lockfile);
      } else {          /* process still running */
	print_log("existing lockfile");
	print_log(lockfile);
	fclose(lckdes);
	free(lockfile);
	return 1;
      }
    } else {
      print_log("error while reading pid from lockfile");
      fclose(lckdes);
      free(lockfile);
      return 1;
    }
  } else if(errno != ENOENT) {
    print_log("existing lockfile, but could not open it");
    free(lockfile);
    return 1;
  }

  lckdes = fopen(lockfile, "w");
  if(! lckdes) {
    print_log("could not create lockfile");
    free(lockfile);
    return 2;
  }

  fprintf(lckdes, "%d", getpid());
  fclose(lckdes);
  free(lockfile);
  return 0;
}

int lock_unlock_file(char *filename)
{
  char *lockfile;

  if(! filename) return 0;

  lockfile = (char *) malloc(sizeof(char) * (strlen(filename) + 6));
  if(! lockfile) {
    perror("malloc");
    exit(1);
  }

  sprintf(lockfile, "%s.lock", filename);

  if(unlink(lockfile)) {
    free(lockfile);
    return 1;
  }
  
  free(lockfile);
  return 0;
}

