/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* mainpart
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 04/01/2000
*/

#include "config.h"

extern int config_verbose;

extern int config_readrc(char *rcfile);
extern int config_include_box(char *box);
extern int config_exclude_box(char *box);
extern int print_log(char *str);
extern int import_main(void);
extern int export_main(void);
extern int send_main(void);

char main_version[80];

void main_usage(char *prg)
{
  fprintf(stderr, "%s\n", main_version);
  fprintf(stderr, "usage: %s [-i|-e|-s|-b|-a|-c config|-v|-h] {+BOX | -BOX}\n"
	          "       -i: do import (getting new mails and import)\n"
		  "       -e: do export (only exporting)\n"
		  "       -s: send exported mails\n"
		  "       -b: do export and send mails\n"
		  "       -a: first do import, then export, then send mails\n"
		  "       -c: use configfile\n"
		  "       -v: verbosity output\n"
		  "       -h: this help\n"
		  "\nFf no configfile is given, \"" SFTOOLRC "\""
		  " is used instead\n\n"
		  "If you include a list of several boxes with prefix \"+\","
		  " everything is only\n"
		  "done for these boxes\n"
		  "Or include a list of several boxes with prefix \"-\", and"
		  " these boxes are\n"
		  "excluded.\n"
		  "Note: You cannot use include and exclude together\n"
		  , prg);
}

int main(int argc, char **argv)
{
  int i;
  char *rc;
  BOOL do_import, do_export, do_send;
  int verbose;
  
  strcpy(main_version, "SF-Tools " VERSION " " __DATE__);

  i         = 1;
  rc        = NULL;
  do_import = FALSE;
  do_export = FALSE;
  do_send   = FALSE;
  verbose   = 0;

  while(i+1 <= argc) {
    if(strcmp(argv[i], "-i") == 0) {
      do_import = TRUE;
      i++;
    } else if(strcmp(argv[i], "-e") == 0) {
      do_export = TRUE;
      i++;
    } else if(strcmp(argv[i], "-s") == 0) {
      do_send = TRUE;
      i++;
    } else if(strcmp(argv[i], "-b") == 0) {
      do_export = TRUE;
      do_send   = TRUE;
      i++;
    } else if(strcmp(argv[i], "-a") == 0) {
      do_import = TRUE;
      do_export = TRUE;
      do_send   = TRUE;
      i++;
    } else if(strcmp(argv[i], "-v") == 0) {
      verbose = 1;
      i++;
    } else if(strcmp(argv[i], "-h") == 0) {
      main_usage(argv[0]);
      exit(0);
    } else if(strcmp(argv[i], "-c") == 0) {
      if(i+1 == argc) {
	fprintf(stderr, "option %s needs filename\n", argv[i]);
	exit(1);
      }
      rc = argv[i+1];
      i += 2;
    } else if(argv[i][0] == '+') {
      if(config_include_box(argv[i]+1)) {
	exit(1);
      }
      i++;
    } else if(argv[i][0] == '-') {
      if(config_exclude_box(argv[i]+1)) {
	exit(1);
      }
      i++;
    } else {
      fprintf(stderr, "invalid option \"%s\"\n", argv[i]);
      main_usage(argv[0]);
      exit(1);
    }
  }
  
  config_readrc(rc);
  config_verbose = verbose;
  
  if((!do_import) && (!do_export) && (!do_send)) {
    print_log("nothing to do. Quit.");
    return 0;
  }
  
  print_log("SF-Tool " VERSION " started");
  if(do_import) import_main();
  if(do_export) export_main();
  if(do_send)   send_main();
  print_log("SF-Tool " VERSION " finished");
  
  return 0;
}

