/* Mime2UU
 * Written by Matthias Hensler <wsp@gmx.de>, 29/9/1999
 * Copyright WSPse 1999
 * Last updated by Matthias Hensler <wsp@gmx.de>, 5/1/2000
 */

/* how to parse an incoming mime-mail from stdin, including a PKzip-archiv
   (with one file inside), decoded to stdout:
   cat <mail> | mime2uu - - | uudecode -o /dev/stdout | gunzip -c > <out>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define MAXLEN 512

int decode(FILE *in, char *oname)
{
  FILE *out;
  char buffer[MAXLEN];
  char name[100];
  char name2[100];
  char *str1, *str2;
  int done;

  done = 0;
  while(1) {
    fgets(buffer, MAXLEN-1, in);
    if(feof(in)) return 1;
    str1 = strstr(buffer, "name=\"");
    if(str1) str1 += 6;
    else {
      str1 = strstr(buffer, "filename=\"");
      if(str1) str1 += 10;
      else {
	str1 = strstr(buffer, "name=");
	if(str1) str1 += 5;
	else {
	  str1 = strstr(buffer, "filename=");
	  if(str1) str1 += 9;
	}
      }
    }
    if(str1) {
      str2 = strchr(str1, '\n');
      if(str2) *str2 = 0;
      str2 = strchr(str1, '\r');
      if(str2) *str2 = 0;
      str2 = strchr(str1, '\"');
      if(str2) *str2 = 0;
      if(*str1) {
	strcpy(name, str1);
	done |= 1;
      }
    }

    if(strstr(buffer, "base64")) done |= 2;

    if(done == 3) break;
  }

  while(1) {
    fgets(buffer, MAXLEN-1, in);
    if(feof(in)) return 1;
    if(buffer[0] == '\n' || buffer[0] == '\r') break;
  }

  strcpy(name2, name);
  fprintf(stderr, "Found base64 file \"%s\"\n", name);
  if(! oname) {
    sprintf(name, "%s.uu", name2);
    if(! access(name, F_OK)) {
      done=0;
      while(! access(name, F_OK)) {
	sprintf(name, "%s.uu-%d", name2, done++);
      }
    }
  } else {
    strcpy(name, oname);
  }

  if(strcmp(name, "-") != 0) {
    fprintf(stderr, "using \"%s\"\n", name);

    out = fopen(name, "w");
    if(! out) {
      fprintf(stderr, "opening failed\n");
      return 1;
    }
  } else {
    out = stdout;
  }

  fprintf(out, "begin-base64 664 %s\n", name2);

  while(1) {
    fgets(buffer, MAXLEN-1, in);
    if(feof(in)) break;
    str1 = strrchr(buffer, '\n');
    if(str1) *str1 = 0;
    str1 = strrchr(buffer, '\r');
    if(str1) *str1 = 0;
    if(strchr(buffer, '=')) break;
    if(buffer && strlen(buffer) > 58) {
      fprintf(out, "%s\n", buffer);
    }
  }

  fprintf(out, "%s\n====\n", buffer);
  fclose(out);

  return 0;
}

int main(int argc, char **argv)
{
  FILE *in;
  char *out;
  
  if(argc < 2 || argc > 3) {
    fprintf(stderr, "usage: %s mimefile [outfile]\n"
	    "if mimefike = -, then stdin is used\n"
	    "if outfile = -, output goes to stdout\n", argv[0]);
    exit(1);
  }

  if(strcmp(argv[1], "-") == 0) {
    in = stdin;
  } else {
    in = fopen(argv[1], "r");
    if(! in) {
      fprintf(stderr, "could not open \"%s\"\n", argv[1]);
      perror("fopen");
      exit(1);
    }
  }

  if(argc == 3) out = argv[2];
  else          out = NULL;
  
  while(! decode(in, out));

  fclose(in);

  return 0;
}

