/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* doing mail sending
   written by Matthias Hensler <wsp@gmx.de>, 28/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 12/11/1999
*/

#include "config.h"
#include "struct.h"

#define MAXLEN 512

extern boxlist *config_boxlist;
extern char    *config_expdir;
extern char    *config_tmpdir;
extern char    *config_copydir;
extern char     config_ownbbs[];
extern char    *config_mailserv;
extern char    *config_mxsender;
extern char    *config_faildir;
extern char    *config_encoder;
extern char    *config_mimeenc[];
extern char    *config_mimesuf[];
extern char    *config_mimecon[];

extern int print_log(char *str);
extern int config_create_sub_dirs(char *filename, BOOL mode);
extern char *spool_find_unique(char *outputdir, char *str);
extern int spool_copy_file(char *inputfile, char *outputdir, char *str);
extern int shell_run_cmd(char *cmd, char *pat1, char *pat2);
extern int open_socket (const char *address);
extern int read_from_sock(char **line, int sock_fd, BOOL wait);
extern int write_to_sock(char *buf, int len, int sock_fd);
extern int config_box_action_allowed(char *box);

int send_create_header(char *mailfile, char *tmpfile, char *mxto, int protokoll, int mime)
{
  char buffer[MAXLEN];
  FILE *in, *out;
  char *str;

  in  = fopen(tmpfile, "r");
  out = fopen(mailfile, "w");

  if((!in) || (!out)) {
    print_log("error creating mailheader");
    return 1;
  }
  
  fprintf(out, "From: %s\n"
	  "Reply-To: %s\n"
	  "To: %s\n"
	  "Subject: S&F [Export-Mailbox=%s, Export-Protokoll=%d]\n"
	  "X-SFtool: WSPse SFtool " VERSION "\n"
	  "X-Problems-To: wsp@gmx.de\n"
	  "X-SFtool-Homepage: excelsior.kullen.rwth-aachen.de\n"
	  , config_mxsender, config_mxsender, mxto, config_ownbbs, protokoll);

  if(mime) {
    fprintf(out, "MIME-Version: 1.0\n"
	    "Content-Type: multipart/mixed; boundary=WSPse.1999.hensler.30.9\n"
	    "\n--WSPse.1999.hensler.30.9\n"
	    "Content-type: text/plain\n"
	    "Content-Transfer-Encoding: quoted-printable\n");
  }
  
  fprintf(out, "\n"
	  "This mail contains an exportfile from your S&F\n"
	  "partner %s. Fileformat is "
	  , config_ownbbs);
  
  switch(protokoll) {
    case 1:
      fprintf(out, "F6FBB");
      break;
    case 2:
      fprintf(out, "WinGT");
      break;
    case 3:
      fprintf(out, "DPbox");
      break;
    default:
      fprintf(out, "unknown");
  }
  
  fprintf(out, "\n\nThe following header is parsed by sftool:\n");
  if(mime) {
    fprintf(out, "Export-Mailbox=3D%s=20\n"
	    "Export-Protokoll=3D%d=20\n"
	    "Export-Mime=3D%d=20\n"
	    , config_ownbbs, protokoll, mime);
  } else {
    fprintf(out, "Export-Mailbox=%s\n"
	    "Export-Protokoll=%d\n"
	    "Export-Mime=%d\n"
	    , config_ownbbs, protokoll, mime);
  }
  fprintf(out, "\nReport any problems with SFtool to wsp@gmx.de\n"
	  "automaticly created by sftool " VERSION "\n"
	  "\n---encoded exportfile follows---\n\n");

  if(mime) {
    fprintf(out, "--WSPse.1999.hensler.30.9\n"
	    "Content-type: %s\n"
	    "        name=\"%s%s\"\n"
	    "Content-Description: Forward von %s\n"
	    "Content-Transfer-Encoding: base64\n"
	    "Content-Disposition: attachment;\n"
	    "        filename=\"%s%s\"\n"
	    "Content-Transfer-Encoding: base64\n\n"
	    , config_mimecon[mime-1], config_ownbbs, config_mimesuf[mime-1], config_ownbbs, config_ownbbs, config_mimesuf[mime-1]);
  }

  while(1) {
    str = fgets(buffer, MAXLEN-1, in);
    if(feof(in)) break;
    if(! str) {
      perror("fgets");
      print_log("fgets failure while writing header");
      break;
    }
    if(fputs(buffer, out) < 0) {
      perror("fputs");
      print_log("fputs failure while writing header");
      break;
    }
  }

  if(mime) {
    fprintf(out, "\n--WSPse.1999.hensler.30.9--\n\n");
  }
  
  fclose(in);
  fclose(out);
	  
  return 0;
}

int send_put_to_sock(int sock_fd, char *str)
{
  return write_to_sock(str, strlen(str), sock_fd);
}

int send_get_from_sock(int sock_fd)
{
  char *line;
  char *line2;
  int ret_val;

  usleep(150000);
  if(read_from_sock(&line, sock_fd, TRUE)) {
    print_log("failure reading from sock");
    return -1;
  }

  line2 = NULL;
  while(! read_from_sock(&line2, sock_fd, FALSE)) {
    if(line) free(line);
    line = line2;
    line2 = NULL;
  }
  
  if(line2) {
    if(line) free(line);
    line = line2;
  }

  if(! line) {
    print_log("no data");
  }

  line2 = strrchr(line, '\n');
  if(line2) *line2 = 0;
  line2 = strrchr(line, '\r');
  if(line2) *line2 = 0;

  if(*line == '2')      ret_val = 2;
  else if(*line == '3') ret_val = 3;
  else {
    ret_val = -2;
    print_log("errorcode from mailserver");
    print_log(line);
    send_put_to_sock(sock_fd, "QUIT\n");
  }
  
  free(line);

  return ret_val;
}

int send_send_mail(char *file, char *mxfrom, char *mxto)
{
  int mx_fd;
  int code;
  char *tmp, *str;
  FILE *mail;
  char buffer[MAXLEN];
  struct stat st;

  if(stat(file, &st)) {
    print_log("unknown mailsize");
  } else {
    sprintf(buffer, "sending mail with size of %ld kb", st.st_size / 1024);
    print_log(buffer);
  }

  mx_fd = open_socket(config_mailserv);
  if(mx_fd < 0) {
    print_log("no connection to mailserver");
    return 1;
  }
  
  code = send_get_from_sock(mx_fd);
  if(code != 2) {
    close(mx_fd);
    return 1;
  }

  tmp = getenv("HOSTNAME");
  if(tmp) code = strlen(tmp) + 10;
  else    code = 20;

  str = (char *) malloc(sizeof(char) * code);
  if(! str) {
    perror("malloc");
    exit(1);
  }
  
  if(tmp) sprintf(str, "EHLO %s\n", tmp);
  else    strcpy(str, "EHLO localhost\n");

  if(send_put_to_sock(mx_fd, str)) {
    print_log("EHLO to mailserver failed");
    free(str);
    close(mx_fd);
    return 1;
  }

  code = send_get_from_sock(mx_fd);
  if(code != 2) {
    close(mx_fd);
    free(str);
    return 1;
  }

  str = (char *) realloc(str, sizeof(char) * (strlen(mxfrom) + 20));
  if(! str) {
    perror("realloc");
    exit(1);
  }

  sprintf(str, "MAIL FROM: <%s>\n", mxfrom);

  if(send_put_to_sock(mx_fd, str)) {
    print_log("MAIL FROM to mailserver failed");
    free(str);
    close(mx_fd);
    return 1;
  }

  code = send_get_from_sock(mx_fd);
  if(code != 2) {
    close(mx_fd);
    free(str);
    return 1;
  }

  str = (char *) realloc(str, sizeof(char) * (strlen(mxto) + 20));
  if(! str) {
    perror("realloc");
    exit(1);
  }

  sprintf(str, "RCPT TO: <%s>\n", mxto);

  if(send_put_to_sock(mx_fd, str)) {
    print_log("RCPT TO to mailserver failed");
    free(str);
    close(mx_fd);
    return 1;
  }

  free(str);
  
  code = send_get_from_sock(mx_fd);
  if(code != 2) {
    close(mx_fd);
    return 1;
  }
  
  if(send_put_to_sock(mx_fd, "DATA\n")) {
    print_log("DATA to mailserver failed");
    close(mx_fd);
    return 1;
  }

  code = send_get_from_sock(mx_fd);
  if(code != 3) {
    close(mx_fd);
    return 1;
  }
  
  mail = fopen(file, "r");
  if(! mail) {
    print_log("could not open maildata file");
    close(mx_fd);
    return 1;
  }
  
  while(1) {
    fgets(buffer, MAXLEN-1, mail);
    if(feof(mail)) break;
    if(strcmp(buffer, ".\n") == 0) {
      code = send_put_to_sock(mx_fd, ". \n");
    } else {
      code = send_put_to_sock(mx_fd, buffer);
    }
    if(code) {
      print_log("failure sending data to server");
      close(mx_fd);
      fclose(mail);
      return 1;
    }
  }
  
  fclose(mail);
  
  if(send_put_to_sock(mx_fd, ".\n")) {
    print_log("error closing mail");
    close(mx_fd);
    return 1;
  }

  code = send_get_from_sock(mx_fd);
  if(code != 2) {
    print_log("mail sending failed");
    code = 1;
  } else {
    code = 0;
    send_put_to_sock(mx_fd, "QUIT\n");
  }
 
  close(mx_fd);
  
  return code;
}

int send_export(char *expfile, char *mxto, char *bbs, int protokoll, int mime)
{
  char *tmpfile;
  char *mailfile;
  int ex_stat;
  char buffer[100];

  if(! mxto) return 1;
  
  sprintf(buffer, "export for %s, type=%d", bbs, protokoll);
  print_log(buffer);

  tmpfile = spool_find_unique(config_tmpdir, "exp");
  if(! tmpfile) return 1;

  if(mime) ex_stat = shell_run_cmd(config_mimeenc[mime-1], expfile, tmpfile);
  else     ex_stat = shell_run_cmd(config_encoder, expfile, tmpfile);
  if(ex_stat) {
    sprintf(buffer, "encoding export failed with code %d", ex_stat);
    spool_copy_file(expfile, config_faildir, "expfail");
    print_log(buffer);
    free(tmpfile);
    return 1;
  }

  mailfile = spool_find_unique(config_tmpdir, "mail");
  if(! mailfile) {
    free(tmpfile);
    return 1;
  }
  
  if(send_create_header(mailfile, tmpfile, mxto, protokoll, mime)) {
    sprintf(buffer, "could not create mailfile");
    free(mailfile);
    free(tmpfile);
    return 1;
  }
  
  unlink(tmpfile);
  free(tmpfile);

  spool_copy_file(mailfile, config_copydir, "export");
  
  ex_stat = send_send_mail(mailfile, config_mxsender, mxto);
  if(ex_stat) {
    print_log("sending mail failed");
    spool_copy_file(mailfile, config_faildir, "mailfail");
  } else {
    print_log("mail successfully sent");
    unlink(expfile);
  }
  
  unlink(mailfile);
  free(mailfile);

  return ex_stat;
}

int send_main(void)
{
  boxlist *curr;
  char *file;
  struct stat st;

  print_log("start sending");

  file = (char *) malloc(sizeof(char) * (strlen(config_expdir) + 40));
  if(! file) {
    perror("malloc");
    exit(1);
  }
  
  curr = config_boxlist;
  while(curr) {
    if(! config_box_action_allowed(curr->bbs)) {
      sprintf(file, "sending for box \"%s\" disallowed by rule", curr->bbs);
      print_log(file);
    } else {
      sprintf(file, "%s/%s.exp", config_expdir, curr->bbs);
      if(access(file, F_OK)) {
	sprintf(file, "no export for %s", curr->bbs);
	print_log(file);
      } else {
	if(stat(file, &st)) {
	  print_log("stat exportfile failed, abort");
	} else {
	  if(st.st_size < 10) {
	    sprintf(file, "empty exportfile for %s", curr->bbs);
	    print_log(file);
	  } else {
	    send_export(file, curr->email, curr->bbs, curr->protokoll, curr->mime);
	  }
	}
      }
    }
    
    curr = curr->next;
  }
  
  free(file);
  print_log("sending finished");
  
  return 0;
}

