/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* running childprocesses
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 29/9/1999
*/

#include "config.h"

#define SHELL_TIMEOUT 180    /* max. runtime for shellcommands */

extern int print_log(char *str);

/* build up argument tree for execv */
char **shell_build_arg_tree(const char *line)
{
  int i,j,k;
  BOOL quote;
  BOOL space;
  char **ret;

  quote = FALSE;   /* Check Quotes and number or args */
  space = TRUE;
  j     = 0;

  if(! line) return(NULL);

  for(i=0;i<strlen(line);i++) {
    if(*(line + i) == '\"') {
      space = FALSE;
      if(! quote) quote=TRUE;
      else {
   	quote=FALSE;
	j++;
	space = TRUE;
      }
    } else if(*(line + i) == ' ') {
      if((! quote) && (!space)) {
	space = TRUE;
	j++;
      }
    } else {
      space = FALSE;
    }
  }

  if(quote) return(NULL);
  if(space) j++;
  
  ret = (char **) malloc(sizeof(char *) * (j+1));
  if(ret == NULL) {
    perror("malloc");
    exit(1);
  }
  
  j = 0;
  k = 0;
  space = TRUE;
  quote = FALSE;
  for(i=0;i<strlen(line);i++) {
    if(*(line + i) == '\"') {
      if(! quote) {
	quote = TRUE;
	k = i+1;
      } else {
	quote = FALSE;
	ret[j] = (char *) malloc(sizeof(char) * (i-k+1));
	if(ret[j] == NULL) {
	  perror("malloc");
	  exit(1);
	}
	*(ret[j]+(i-k)) = 0;
	if(i-k > 0) {
	  memcpy((ret[j]), (line + k), i-k);
	}
	j++;
	space = TRUE;
      }
    } else if(*(line + i) == ' ') {
      if((! space) && (! quote)) {
	ret[j] = (char *) malloc(sizeof(char) * (i-k+1));
	if(ret[j] == NULL) {
	  perror("malloc");
	  exit(1);
	}
	*(ret[j]+(i-k)) = 0;
	if(i-k > 0) {
	  memcpy((ret[j]), (line + k), i-k);
	}
	j++;
	space = TRUE;
      }
    } else {
      if(space) k = i;
      space = FALSE;
    }
  }
 
  if(! space) {
    ret[j] = (char *) malloc(sizeof(char) * (i-k+1));
    if(ret[j] == NULL) {
      perror("malloc");
      exit(1);
    }
    *(ret[j]+(i-k)) = 0;
    if(i-k > 0) {
      memcpy((ret[j]), (line + k), i-k);
    }
    j++;
  }
  ret[j] = NULL;
  return(ret);
}

int shell_run_cmd(char *cmd, char *pat1, char *pat2)
{
  char *fn_new;
  char *add_poi;
  char def_add[2];
  int i,j,k;
  BOOL pat;
  pid_t child;
  time_t start_time;
  char **cmd_args;
  int max_sek;

  if(! cmd) return 1;

  if(pat2) max_sek = SHELL_TIMEOUT;
  else     max_sek = 900;

  fn_new = (char *) malloc(sizeof(char));
  if(! fn_new) {
    perror("malloc");
    exit(1);
  }
  *fn_new = 0;
  
  i   = 0;
  pat = FALSE;
  for(j=0;j<strlen(cmd);j++) {
    if(pat) {
      pat = FALSE;
      k   = j-1-i;
      if(k > 0) {
	fn_new = (char *) realloc(fn_new, sizeof(char) * (strlen(fn_new) + k + 1));
	if(! fn_new) {
	  perror("realloc");
	  exit(1);
	}
	*(fn_new + strlen(fn_new) + k) = 0;
	memcpy((fn_new + strlen(fn_new)), (cmd + i), k);
      }
      i = j+1;
      switch(*(cmd + j)) {
	case '%':
	  def_add[0] = '%';
	  def_add[1] = 0;
	  add_poi = def_add;
	  break;
	case '1':
	  add_poi = pat1;
	  break;
	case '2':
	  add_poi = pat2;
	  break;
	default:
	  def_add[0] = *(cmd + j);
	  def_add[1] = 0;
	  add_poi    = def_add;
      }
      if(! def_add) k = 0;
      else          k = strlen(add_poi);
      if(k > 0) {
	fn_new = (char *) realloc(fn_new, sizeof(char) * (strlen(fn_new) + k + 1));
	if(! fn_new) {
	  perror("realloc");
	  exit(1);
	}
	strcat(fn_new, add_poi);
      }
    } else if(*(cmd + j) == '%') pat = TRUE;
  }
  k = j-i;
  if(k>0) {
    fn_new = (char *) realloc(fn_new, sizeof(char) * (strlen(fn_new) + k + 1));
    if(! fn_new) {
      perror("realloc");
      exit(1);
    }
    *(fn_new + strlen(fn_new) + k) = 0;
    memcpy((fn_new + strlen(fn_new)), (cmd + i), k);
  }
	  
  if(! fn_new) return 1;

  start_time = time(NULL);
  child      = fork();

  if(child == -1) {
    print_log("forking failed");
    free(fn_new);
    return 1;
  }

  if(child == 0) {
    cmd_args = shell_build_arg_tree(fn_new);
    if(cmd_args) {
      execvp(cmd_args[0], cmd_args);
    }
    exit(EXIT_FAILURE);
  }

  /* child is (maybe) running */
  free(fn_new);
  
  while(1) {
    if(waitpid(child, &i, WNOHANG) == child) break;
    sleep(2);
    if(time(NULL) - start_time > max_sek) {
      kill(child, SIGINT);
      print_log("shell cmd timed out, killing child");
      sleep(1);
      kill(child, SIGKILL);
      return 1;
    }
  }

  if(WEXITSTATUS(i) == EXIT_SUCCESS) return 0;
  print_log("child died with error");

  return i;
}

