/* Copying:
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU Gerneral Public License as published by the Free Soft-
   ware Foundation; either version 2 of License, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILTY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 675 Mass
   Ave, Cambridge, MA 02139, USA.
 */

/* get mails from spool
   written by Matthias Hensler <wsp@gmx.de>, 27/9/1999
   Copyright WSPse 1999
   Last updated by Matthias Hensler <wsp@gmx.de>, 5/1/2000
*/

#include "config.h"

#define MAXLEN 256      /* size of copy buffer */

extern char *config_mxspool;
extern char *config_tmpdir;
extern char *config_copydir;

extern int print_log(char *str);
extern int lock_lock_file(char *filename);
extern int lock_lock_file(char *filename);
extern int lock_unlock_file(char *filename);
extern int config_create_sub_dirs(char *filename, BOOL mode);
extern int import_file_import(char *file);

typedef struct tmp {
  char *name;
  struct tmp *next;
} filelist;

char *spool_find_unique(char *outputdir, char *str)
{
  char *pat, *outfile;
  time_t lt;
  struct tm *ltm;
  int i;
 
  if((! outputdir) || (! *outputdir) || strcmp(outputdir, "0") == 0 || (! str)) return NULL;
  
  config_create_sub_dirs(outputdir, FALSE);

  pat     = (char *) malloc(sizeof(char) * (strlen(str) + 40));
  outfile = (char *) malloc(sizeof(char) * (strlen(outputdir) + strlen(str) + 40));
  if((! pat) || (! outfile)) {
    perror("malloc");
    exit(1);
  }

  lt  = time(NULL);
  ltm = localtime(&lt);

  sprintf(pat, "%%s/%s-%04d%02d%02d-%%d", str, 1900 + ltm->tm_year, ltm->tm_mon+1, ltm->tm_mday);
  
  i = 1;
  while(i) {
    sprintf(outfile, pat, outputdir, i);
    if(access(outfile, F_OK)) break;
    i++;
  }

  free(pat);
  return outfile;
}
  
int spool_copy_file(char *inputfile, char *outputdir, char *str)
{
  char *outfile;
  int c_in, c_out;
  char buffer[MAXLEN];
  size_t len;
  
  if((! outputdir) || (! *outputdir) || strcmp(outputdir, "0") == 0 || (! str) ||
       (! inputfile) || (! *inputfile)) return 1;

  outfile = spool_find_unique(outputdir, str);
  if(! outfile) return 1;

  c_in  = open(inputfile, O_RDONLY);
  c_out = open(outfile, O_WRONLY | O_CREAT, 0644);

  if(c_in < 0 || c_out < 0) {
    free(outfile);
    return 1;
  }

  while(1) {
    len = read(c_in, &buffer, MAXLEN);
    if(len == 0) break;
    if(write(c_out, buffer, len) != len) {
      perror("write");
      print_log("copying failed");
      break;
    }
  }

  close(c_out);
  close(c_in);
  free(outfile);
  
  return 0;
}

int spool_get_mails(void)
{
  int lckstat;
  filelist *fl_anc, *fl_new;
  FILE *spool_in;
  FILE *spool_out;
  struct stat st;
  char buffer[MAXLEN];
  int sp_stat;
  BOOL had_output;

  lckstat = lock_lock_file(config_mxspool);
  
  if(lckstat) {
    print_log("locking mailbox failed");
    if(lckstat != 2) {
      print_log("abort import");
      return 1;
    }
  }
  
  if(access(config_mxspool, F_OK)) {
    print_log("no new mails, quit");
    if(! lckstat) lock_unlock_file(config_mxspool);
    return 0;
  }
  
  if(stat(config_mxspool, &st)) {
    print_log("could not stat mailbox, abort");
    if(! lckstat) lock_unlock_file(config_mxspool);
    return 1;
  }

  if(st.st_size < 10) {
    print_log("mailbox empty, quit");
    if(! lckstat) lock_unlock_file(config_mxspool);
    return 0;
  }
  
  spool_in = fopen(config_mxspool, "r");
  if(! spool_in) {
    print_log("could not open mailbox");
    if(! lckstat) lock_unlock_file(config_mxspool);
    return 1;
  }
  
  fl_anc = NULL;
  
  buffer[0] = 0;
  sp_stat   = 0;
  
  while(1) {
    if(feof(spool_in)) break;
    
    fl_new = (filelist *) malloc(sizeof(filelist));
    if(! fl_new) {
      perror("malloc");
      exit(1);
    }

    fl_new->next = fl_anc;
    fl_anc       = fl_new;
    fl_new->name = spool_find_unique(config_tmpdir, "spool");

    if(! fl_new->name) {
      print_log("no name for tempfile");
      sp_stat = 1;
      fl_anc = fl_new->next;
      break;
    }
    
    spool_out  = fopen(fl_new->name, "w");
    had_output = FALSE;
    if(! spool_out) {
      print_log("could not open tmpfile");
      sp_stat = 1;
      free(fl_new->name);
      fl_anc = fl_new->next;
      break;
    }

    if(buffer[0]) fputs(buffer, spool_out);
    
    while(1) {
      fgets(buffer, MAXLEN-1, spool_in);
      if(feof(spool_in)) {
	fclose(spool_out);
	buffer[0] = 0;
	break;
      }

      /* I don't like this very much, but it should be enough for this little tool
       * (Note: I don't want to reprogram procmail ;-) */
      if(strncmp(buffer, "From ", 5) == 0 && had_output) {
	fclose(spool_out);
	break;
      }
      fputs(buffer, spool_out);
      had_output = TRUE;
    }
  }

  fclose(spool_in);
  
  if(! sp_stat) {
    spool_in = fopen(config_mxspool, "w+");
    if(spool_in) fclose(spool_in);
    if((!spool_in) && unlink(config_mxspool)) {
      print_log("warning: could not clear mailbox!");
    } else {
      print_log("mailbox cleared");
    }
  }
  print_log("closing mailbox");
  if(! lckstat) lock_unlock_file(config_mxspool);
  
  fl_new = fl_anc;
  while(fl_new) {
    import_file_import(fl_new->name);
    spool_copy_file(fl_new->name, config_copydir, "import");
    unlink(fl_new->name);
    free(fl_new->name);
    fl_anc = fl_new;
    fl_new = fl_new->next;
    free(fl_anc);
  }

  return 0;
}

